#include <stdexcept>
#include "decoder.h"

Decoder::Decoder(std::string file)
{
    formatContext = avformat_alloc_context();
    if(avformat_open_input(&formatContext, file.c_str(), nullptr, nullptr) != 0)
        throw std::runtime_error("Cannot open file: " + file);
    if(avformat_find_stream_info(formatContext, nullptr) < 0)
        throw std::runtime_error("Cannot find stream info in file: " + file);
    AVCodec *codec;
    auto videoStreamId = av_find_best_stream(formatContext, AVMEDIA_TYPE_VIDEO, -1, -1, const_cast<const AVCodec **>(&codec), 0);
    if(videoStreamId < 0)
        throw std::runtime_error("No video stream available");
    if(!codec)
        throw std::runtime_error("No suitable codec found");
    codecContext = avcodec_alloc_context3(codec);
    if(!codecContext)
        throw std::runtime_error("Cannot allocate codec context memory");
    if(avcodec_parameters_to_context(codecContext, formatContext->streams[videoStreamId]->codecpar) < 0)
        throw std::runtime_error{"Cannot use the file parameters in context"};
    if(avcodec_open2(codecContext, codec, nullptr) < 0)
        throw std::runtime_error("Cannot open codec.");
}

AVFrame *Decoder::getFrame()
{
    frame = av_frame_alloc();
    AVPacket *packet = av_packet_alloc();
    av_read_frame(formatContext, packet);
    avcodec_send_packet(codecContext, packet);
    avcodec_send_packet(codecContext, nullptr);
    int err = avcodec_receive_frame(codecContext, frame);
    if(err < 0)
		throw std::runtime_error("Cannot receive frame");
	av_packet_free(&packet);
	return frame;
}

Decoder::~Decoder()
{
    avformat_close_input(&formatContext);
    avformat_free_context(formatContext);
    avcodec_free_context(&codecContext);
    av_frame_free(&frame);
}
    
